# -*- coding: utf-8 -*-
# @Time    : 2024/12/24 13:51
# @Author  : xuwei
# @FileName: log.py
# @Software: PyCharm

import os
import logging
from logging.handlers import RotatingFileHandler


# 在第一次使用的时候才会初始化 logger，self.__init_logger()
class Log:
    def __init__(self):
        """
        3种log handler：1- 输出到文件、输出到控制台、ddp输出到graylog，3个handler可以同时开启
        """
        self.file_handler = True
        self.log_path = './'
        self.name = 'test'  # 决定log文件名称 test.log

        self.console_handler = False

        self.graylog_handler = False
        self.graylog_udp_host = ""
        self.graylog_udp_port = 0

        self.log_level = logging.INFO
        self.log_format = False

        self.__init_log = False

    ## 根据 init_log 修改 self, 只修改一次
    def __init_logger(self):
        if self.__init_log: return
        self.__init_log = True

        self.__logger = logging.getLogger(self.name)
        self.__logger.setLevel(self.log_level)

        formatter = logging.Formatter('[%(levelname)s-%(process)s][%(asctime)s]%(message)s')
        # 确定用哪个handler
        if self.graylog_handler and self.graylog_udp_host:
            import graypy
            graypy_handler = graypy.GELFUDPHandler(self.graylog_udp_host, self.graylog_udp_port)

            graypy_handler.setFormatter(formatter)
            graypy_handler.setLevel(self.log_level)
            self.__logger.addHandler(graypy_handler)

        if self.file_handler:
            log_name = os.path.join(self.log_path, '%s.log' % self.name)
            # file_handler = logging.FileHandler(log_name, 'a', encoding='utf-8')

            # 根据文件大小切割，创建 RotatingFileHandler
            file_handler = logging.handlers.RotatingFileHandler(
                filename=log_name,  # 日志文件名
                maxBytes=1024 * 1024 * 5,  # 1024单个日志文件的最大字节数（1KB）, 1024 * 1024 每个文件最大 1MB
                backupCount=5,  # 最多保留 5 个日志文件
                encoding='utf-8'
            )

            file_handler.setFormatter(formatter)
            file_handler.setLevel(self.log_level)
            self.__logger.addHandler(file_handler)

        if self.console_handler:
            # 创建一个StreamHandler,用于输出到控制台
            console_handler = logging.StreamHandler()

            console_handler.setFormatter(formatter)
            console_handler.setLevel(self.log_level)
            self.__logger.addHandler(console_handler)

    def __console(self, level, message):
        self.__init_logger()

        data = message
        if self.name and self.log_format:
            data = {'facility': self.name, 'log_level': level, 'msg': message}
        if level == 'info':
            self.__logger.info(data)
        elif level == 'debug':
            self.__logger.debug(data)
        elif level == 'warning':
            self.__logger.warning(data)
        elif level == 'error':
            self.__logger.error(data)
        elif level == 'except':
            self.__logger.exception(data)

    def debug(self, message):
        self.__console('debug', message)

    def info(self, message):
        self.__console('info', message)

    def warning(self, message):
        self.__console('warning', message)

    def error(self, message):
        self.__console('error', message)

    def exception(self, message):
        self.__console('except', message)

    def raise_exception(self, message):
        self.exception(message)
        raise Exception(message)


log = Log()


def init_log(
        file_handler=True,
        log_path='./',
        name='log',

        console_handler=False,

        graylog_handler=False,
        graylog_udp_host: str = "",
        graylog_udp_port: int = 0,

        log_level=logging.INFO,
        log_format=False,
):
    """
    file_path: 存日志文件的文件夹路径
    name: 可以看做是服务的名称，修改name参数，日志文件的前缀名称，默认是 log.error.log，log.info.log
    log_format: 日志输出的格式是否格式化，格式化后是个json [ERROR][2024-01-24 14:04:21,693]{'name': 'test.', 'log_level': 'error', 'msg': 'this is a test2'}
    graylog_udp_host, graylog_udp_port: 为真的话，日志不写文件，而是写入graylog中
    """
    if not os.path.exists(log_path):
        os.mkdir(log_path)

    log.file_handler = file_handler
    log.log_path = log_path
    log.name = name

    log.console_handler = console_handler

    log.graylog_handler = graylog_handler
    log.graylog_udp_host = graylog_udp_host
    log.graylog_udp_port = graylog_udp_port

    log.log_level = log_level
    log.log_format = log_format



# -*- coding: utf-8 -*-
# @Time    : 2025/2/21 09:46
# @Author  : xuwei
# @FileName: response.py
# @Software: PyCharm


from pydantic import BaseModel
from typing import Optional, Generic, TypeVar

T = TypeVar('T')

CODE_SYSTEM_ERROR = -1
CODE_OK = 0
CODE_SHOW = 1


class BaseResponse(BaseModel, Generic[T]):
    code: int
    message: str
    data: Optional[T] = None



# -*- coding: utf-8 -*-
# @Time    : 2024/12/24 16:50
# @Author  : xuwei
# @FileName: es_op.py
# @Software: PyCharm

from elasticsearch_dsl import async_connections
from elasticsearch_dsl import AsyncSearch
from .dotenv_config import config

"""
Elasticsearch DSL 是一个高级库，旨在帮助编写和运行针对 Elasticsearch 的查询。它建立在官方低级客户端 (elasticsearch) 之上。

该代码只简单封装了es连接建立，通过编写body实现异步查询
"""

class EsOpAsync:
    def __init__(self, label, alias="default", timeout=20, config_map=None):
        self.__alias = alias
        self.__config_map = config_map
        self.__label = label
        self.__timeout = timeout
        
    def init(self):

        if self.__config_map:
            hosts = self.__config_map['host'].split(",")  # 英文逗号分割
            user = self.__config_map.get('user', '')
            password = self.__config_map.get('pass', '')
        else:
            hosts = config(self.__label, 'host').split(",") # 英文逗号分割
            user = config(self.__label, 'user', '')
            password = config(self.__label, 'pass', '')

        async_connections.create_connection(
            alias=self.__alias,
            hosts=hosts,
            http_auth=(user, password),
            timeout=self.__timeout
        )

    async def query(self, index: str, body: dict):
        """
        :param index:
        :param body: body = {"query": {"match": {"title": title}}, "from": 0, "size": 1}
        :return:
        """
        s = AsyncSearch(index=index).from_dict(body).filter().using(self.__alias)
        response = await s.execute()
        res = response.to_dict()
        return res

    async def close(self):
        es = async_connections.get_connection(self.__alias)
        await es.close()



# -*- coding: utf-8 -*-
# @Time    : 2024/12/25 14:05
# @Author  : xuwei
# @FileName: redis_op.py
# @Software: PyCharm

from redis.asyncio import StrictRedis, ConnectionPool
from .dotenv_config import config

class RedisAsyncOp:
    def __init__(self, label='redis', db=0, strict=True, config_map=None, max_connections=5):
        self.__label = label
        self.__db = db
        self.__config_map = config_map
        self.__strict = strict
        self.__max_connections = max_connections
        self.__redis_pool = None
        self.con = StrictRedis

    def init(self):
        """
        使用方法

        redis_op = at.RedisAsyncOp('redis', 3, max_connections=1)
        redis_op.init()

        keys = await redis_op.con.set('stock_company', 'xwewrwrwe')
        print(keys)
        val = await redis_op.con.get('stock_company')
        print(val)

        await redis_op.close()
        """

        if self.__config_map:
            redis_host = self.__config_map['host']
            redis_port = int(self.__config_map['port'])
            redis_pass = self.__config_map.get('pass', "")
        else:
            redis_host = config(self.__label, 'host')
            redis_port = int(config(self.__label, 'port'))
            redis_pass = config(self.__label, 'pass', '')

        if not redis_pass: redis_pass = None

        redis_pool = ConnectionPool(
            host=redis_host, port=redis_port, password=redis_pass, db=self.__db, max_connections=self.__max_connections,
            decode_responses=self.__strict
        )

        self.__redis_pool = redis_pool
        self.con = StrictRedis(connection_pool=self.__redis_pool)

    async def close(self):
        await self.con.close(self.__redis_pool)



# -*- coding: utf-8 -*-
# @Time    : 2025/2/19 15:36
# @Author  : xuwei
# @FileName: dotenv_config.py
# @Software: PyCharm


from dotenv import load_dotenv
import os

_env_loaded = False

_config_file = os.getenv("CONFIG_FILE", default=None)
print("dotenv_path: ", _config_file)


def load_env_once():
    global _env_loaded
    if not _env_loaded:
        load_dotenv(dotenv_path=_config_file)
        _env_loaded = True


# 只会加载一次
load_env_once()


def config(label: str, key: str, default=None):
    # label是.evn中的前缀，类似于APP_xxx
    return os.getenv(label.upper() + "_" + key.upper(), default=default)



# -*- coding: utf-8 -*-
# @Time    : 2024/12/24 11:14
# @Author  : xuwei
# @FileName: __init__.py.py
# @Software: PyCharm


from .dotenv_config import config
from .es_async import EsOpAsync
from .json_p import dump_json,load_json
from .logger import log, init_log
from .redis_async import RedisAsyncOp
from .try_cache import catch_error_log




from setuptools import setup

requirements = [
    'elasticsearch-dsl[async]'
]

with open("README.md", "r", encoding="utf8") as fh:
    long_description = fh.read()

setup(
    name='asy_pt',
    version='0.0.1',
    packages=[
        "asy_pt",
    ],
    license='BSD License',  # example license
    description='asy_pt',
    long_description='这是一个通用的fastapi 开发工具包，帮助你快速的开发fastapi项目',
    install_requires=requirements,
    long_description_content_type="text/markdown",
    url='https://github.com/xulehexuwei',
    author='xuwei',
    author_email='15200813194@163.com',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.10',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# -*- coding: utf-8 -*-
# @Time    : 2024/12/24 14:14
# @Author  : xuwei
# @FileName: try_cache.py
# @Software: PyCharm



from functools import wraps
from .logger import log

def catch_error_log(func):
    @wraps(func)
    def try_catch(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            # print("%s: %s" % (func.__name__, e))
            log.exception("%s: %s" % (func.__name__, e))
            return None

    return try_catch


# -*- coding: utf-8 -*-
# @Time    : 2024/12/25 14:36
# @Author  : xuwei
# @FileName: json_p.py
# @Software: PyCharm

import json


##datetime.datetime is not JSON serializable 报错问题解决
class CJsonEncoder(json.JSONEncoder):
    def default(self, obj):

        from datetime import date
        import datetime

        if obj != obj:
            return None
        elif isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime("%Y-%m-%d")
        # elif isinstance(obj, numpy.integer):
        #     return int(obj)
        # elif isinstance(obj, numpy.floating):
        #     return float(obj)
        # elif isinstance(obj, numpy.ndarray):
        #     return obj.tolist()
        elif isinstance(obj, bytes):
            return str(obj)
        # elif isinstance(obj, bson.ObjectId):
        #     return str(obj)
        return json.JSONEncoder.default(self, obj)


def dump_json(data):
    return json.dumps(data, ensure_ascii=False, cls=CJsonEncoder)


def load_json(data):
    return json.loads(data)
